"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chunk_1 = __importDefault(require("lodash/chunk"));
const math_gl_1 = require("math.gl");
const linePattern = /^(v|vn|uv|f) (.*)$/;
const valuePattern = /([^ ]+)*/g;
const indexListPattern = /^([0-9]*)\/([0-9]*)\/([0-9]*)$/;
function parseIndex(data) {
    if (!data)
        return null;
    return Number(data) - 1;
}
function parseFace(data) {
    const m = data.match(valuePattern);
    if (!m)
        throw new Error('Parse error');
    return chunk_1.default(m, 2).map(a => a[0]).map((value) => {
        const n = value.match(indexListPattern);
        if (!n)
            throw new Error('Parse error');
        const vertex = parseIndex(n[1]);
        const uv = parseIndex(n[2]);
        const normal = parseIndex(n[3]);
        return {
            normal,
            uv,
            vertex,
        };
    });
}
function parseVectors(pattern) {
    return (data) => {
        const m = data.match(pattern);
        if (!m)
            throw new Error('Parse error');
        return m.slice(1).map(value => Number(value));
    };
}
const parseVector2s = parseVectors(/^([0-9\-.]+) ([0-9\-.]+)$/);
const parseVector3s = parseVectors(/^([0-9\-.]+) ([0-9\-.]+) ([0-9\-.]+)$/);
function indexToOBJIndex(index) {
    if (index === null)
        return '';
    return `${index + 1}`;
}
class IndexedModel {
    constructor(faces, vertices, normals, uvs) {
        this.faces = faces;
        this.vertices = vertices;
        this.normals = normals;
        this.uvs = uvs;
    }
    toOBJ(name) {
        const vertices = this.vertices.map(v => `v ${v.x} ${v.y} ${v.z}`);
        const normals = this.normals.map(v => `vn ${v.x} ${v.y} ${v.z}`);
        const uvs = this.uvs.map(v => `uv ${v.x} ${v.y}`);
        const faces = this.faces.map((face) => {
            const indexLists = face.map(indexList => `${indexToOBJIndex(indexList.vertex)}/${indexToOBJIndex(indexList.uv)}/${indexToOBJIndex(indexList.normal)}`);
            return `f ${indexLists.join(' ')}`;
        });
        return [
            `o ${name}`,
            ...vertices,
            ...normals,
            ...uvs,
            ...faces,
        ].join('\n');
    }
    static parseOBJ(obj) {
        const faces = [];
        const normals = [];
        const uvs = [];
        const vertices = [];
        const lines = obj.split(/\n/g);
        lines.forEach((line) => {
            const m = line.match(linePattern);
            if (!m)
                return;
            const type = m[1];
            const data = m[2];
            switch (type) {
                case 'v':
                    vertices.push(new math_gl_1.Vector3(parseVector3s(data)));
                    break;
                case 'vn':
                    normals.push(new math_gl_1.Vector3(parseVector3s(data)));
                    break;
                case 'uv':
                    uvs.push(new math_gl_1.Vector2(parseVector2s(data)));
                    break;
                case 'f':
                    faces.push(parseFace(data));
                    break;
                default:
                    break;
            }
        });
        return new IndexedModel(faces, vertices, normals, uvs);
    }
}
exports.default = IndexedModel;
//# sourceMappingURL=IndexedModel.js.map